var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
define(["require", "exports", "./Composable.slots", "./slots", "@uifabricshared/foundation-settings"], function (require, exports, Composable_slots_1, slots_1, foundation_settings_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function _validateComposable(options) {
        var numSlots = (options.slots && Object.getOwnPropertyNames(options.slots).length) || 0;
        if (!numSlots) {
            throw 'A composable component must have at least one slot specified';
        }
        else if (numSlots > 1) {
            if (!options.render) {
                throw 'A composable component with multiple slots cannot use the default render implementation';
            }
            else if (!options.usePrepareProps) {
                throw 'A composable component with multiple slots cannot use the default usePrepareProps implementation';
            }
        }
    }
    function atomicRender(Slots, _renderData) {
        var children = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            children[_i - 2] = arguments[_i];
        }
        return slots_1.renderSlot.apply(void 0, [Slots.root, undefined].concat(children));
    }
    exports.atomicRender = atomicRender;
    function atomicUsePrepareProps(props, useStyling) {
        var slotProps = foundation_settings_1.mergeSettings(useStyling(props), { root: props });
        return { slotProps: slotProps };
    }
    exports.atomicUsePrepareProps = atomicUsePrepareProps;
    /**
     * Create a component that can be composed into other objects to remove extra levels from the tree
     *
     * @param options - composable options which define the behavior of the component
     */
    function composable(options) {
        // create the functional component
        if (!options.useStyling) {
            options.useStyling = function () {
                return {};
            };
        }
        // ensure we are correctly configured
        _validateComposable(options);
        // use atomic handlers for usePrepareProps / render if necessary
        options.render = options.render || atomicRender;
        options.usePrepareProps = options.usePrepareProps || atomicUsePrepareProps;
        // create the actual implementation
        var render = function (userProps) {
            // split out children, they will be excluded from the prop preparation phase
            var _a = userProps, children = _a.children, props = __rest(_a, ["children"]);
            // prepare the props, all the way down the tree, also build the slots
            var _b = Composable_slots_1.useCompoundPrepare(props, options), renderData = _b.renderData, Slots = _b.Slots;
            // now do the render, adding the children back in
            return options.render.apply(options, [Slots, renderData].concat(children));
        };
        render.__composable = options;
        return render;
    }
    exports.composable = composable;
    /**
     * Helper to create a composable implementation of a simple atomic component
     *
     * @param target - slot type to create an atomic component from
     * @param usePrepareProps - prop processing implementation.
     * @param filter - optional filter.  If set it allows stripping properties before they are passed to target
     */
    function atomic(target, usePrepareProps, filter) {
        return composable({
            usePrepareProps: usePrepareProps,
            slots: { root: { slotType: target, filter: filter } },
            render: atomicRender
        });
    }
    exports.atomic = atomic;
});
//# sourceMappingURL=Composable.js.map